@@ -173,9 +173,9 @@ class JackTrip : public QObject
      * \param plugin Pointer to ProcessPlugin Class
      */
     // void appendProcessPlugin(const std::tr1::shared_ptr<ProcessPlugin> plugin);
-    virtual void appendProcessPluginToNetwork(ProcessPlugin* plugin);
-    virtual void appendProcessPluginFromNetwork(ProcessPlugin* plugin);
-    virtual void appendProcessPluginToMonitor(ProcessPlugin* plugin);
+    virtual void appendProcessPluginToNetwork(QSharedPointer<ProcessPlugin>& plugin);
+    virtual void appendProcessPluginFromNetwork(QSharedPointer<ProcessPlugin>& plugin);
+    virtual void appendProcessPluginToMonitor(QSharedPointer<ProcessPlugin>& plugin);
 
     /// \brief Start the processing threads
     virtual void startProcess(
@@ -219,9 +219,18 @@ class JackTrip : public QObject
         createHeader(mPacketHeaderType);
     }
     /// \brief Sets (override) Buffer Queue Length Mode after construction
-    virtual void setBufferQueueLength(int BufferQueueLength)
+    virtual void setBufferQueueLength(int queueBuffer)
     {
-        mBufferQueueLength = BufferQueueLength;
+        if (mBufferQueueLength == queueBuffer) {
+            return;
+        }
+        mBufferQueueLength = queueBuffer;
+        if (mReceiveRingBuffer != nullptr
+            && (mBufferStrategy == 3 || mBufferStrategy == 4)) {
+            // mReceiveRingBuffer should be an instance of Regulator when mBufferStrategy
+            // is 3 or 4
+            mReceiveRingBuffer->setQueueBufferLength(mBufferQueueLength);
+        }
     }
     virtual void setBufferStrategy(int BufferStrategy)
     {
@@ -542,6 +551,10 @@ class JackTrip : public QObject
         return (mAudioInterface == nullptr) ? false
                                             : mAudioInterface->getHighLatencyFlag();
     }
+    double getLatency() const
+    {
+        return mReceiveRingBuffer == nullptr ? -1 : mReceiveRingBuffer->getLatency();
+    }
     //@}
     //------------------------------------------------------------------------------------
 
@@ -705,11 +718,11 @@ class JackTrip : public QObject
     JackTrip::hubConnectionModeT
         mHubConnectionModeT;  ///< Hub Server Jack Audio Patch Connection Mode
 
-    QVector<ProcessPlugin*>
+    QVector<QSharedPointer<ProcessPlugin> >
         mProcessPluginsFromNetwork;  ///< Vector of ProcessPlugin<EM>s</EM>
-    QVector<ProcessPlugin*>
+    QVector<QSharedPointer<ProcessPlugin> >
         mProcessPluginsToNetwork;  ///< Vector of ProcessPlugin<EM>s</EM>
-    QVector<ProcessPlugin*>
+    QVector<QSharedPointer<ProcessPlugin> >
         mProcessPluginsToMonitor;  ///< Vector of ProcessPlugin<EM>s</EM>
     QTimer mTimeoutTimer;
     QTimer mRetryTimer;
