@@ -76,7 +76,7 @@ void Meter::init(int samplingRate, int bufferSize)
 {
     ProcessPlugin::init(samplingRate, bufferSize);
 
-    fs = float(fSamplingFreq);
+    fs = float(mSampleRate);
     for (int i = 0; i < mNumChannels; i++) {
         static_cast<meterdsp*>(meterP[i])->init(fs);
     }
