@@ -224,6 +224,13 @@ void RingBuffer::readBroadcastSlot(int8_t* ptrToReadSlot)
     std::memset(ptrToReadSlot, 0, mSlotSize);
 }
 
+//*******************************************************************************
+// Not supported in RingBuffer
+void RingBuffer::setQueueBufferLength([[maybe_unused]] int queueBuffer)
+{
+    return;
+}
+
 //*******************************************************************************
 void RingBuffer::setUnderrunReadSlot(int8_t* ptrToReadSlot)
 {
@@ -319,3 +326,10 @@ void RingBuffer::updateReadStats()
     mUnderrunsNew = 0;
     mLevel        = std::ceil(mLevelCur);
 }
+
+//*******************************************************************************
+// Not supported in RingBuffer
+double RingBuffer::getLatency() const
+{
+    return -1;
+}
