@@ -220,6 +220,8 @@ void UdpHubListener::start()
         mAuth.reset(new Auth(mCredsFile, true));
     }
 
+    startOscServer();
+
     cout << "JackTrip HUB SERVER: Waiting for client connections..." << endl;
     cout << "JackTrip HUB SERVER: Hub auto audio patch setting = " << mHubPatch << " ("
          << mHubPatchDescriptions.at(mHubPatch).toStdString() << ")" << endl;
@@ -364,6 +366,19 @@ void UdpHubListener::stopCheck()
     }
 }
 
+void UdpHubListener::queueBufferChanged(int queueBufferSize)
+{
+    cout << "Updating queueBuffer to " << queueBufferSize << endl;
+    QMutexLocker lock(&mMutex);
+    mBufferQueueLength = queueBufferSize;
+    // Now that we have our actual port, remove any duplicate workers.
+    for (int i = 0; i < gMaxThreads; i++) {
+        if (mJTWorkers->at(i) != nullptr) {
+            mJTWorkers->at(i)->setBufferQueueLength(mBufferQueueLength);
+        }
+    }
+}
+
 //*******************************************************************************
 // Returns 0 on error
 int UdpHubListener::readClientUdpPort(QSslSocket* clientConnection, QString& clientName)
